package at.ac.uibk.gitsearch.service;

import at.ac.uibk.gitsearch.domain.ReviewRating;
import at.ac.uibk.gitsearch.repository.ReviewRatingRepository;
import at.ac.uibk.gitsearch.service.dto.ReviewRatingDTO;
import at.ac.uibk.gitsearch.service.mapper.ReviewRatingMapper;
import java.util.LinkedList;
import java.util.List;
import java.util.Optional;
import java.util.stream.Collectors;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

/**
 * Service Implementation for managing {@link ReviewRating}.
 */
@Service
@Transactional
public class ReviewRatingService {

    private final Logger log = LoggerFactory.getLogger(ReviewRatingService.class);

    private final ReviewRatingRepository reviewRatingRepository;

    private final ReviewRatingMapper reviewRatingMapper;

    public ReviewRatingService(ReviewRatingRepository reviewRatingRepository, ReviewRatingMapper reviewRatingMapper) {
        this.reviewRatingRepository = reviewRatingRepository;
        this.reviewRatingMapper = reviewRatingMapper;
    }

    /**
     * Save a reviewRating.
     *
     * @param reviewRatingDTO the entity to save.
     * @return the persisted entity.
     */
    public ReviewRatingDTO save(ReviewRatingDTO reviewRatingDTO) {
        log.debug("Request to save ReviewRating : {}", reviewRatingDTO);
        ReviewRating reviewRating = reviewRatingMapper.toEntity(reviewRatingDTO);
        reviewRating = reviewRatingRepository.save(reviewRating);
        return reviewRatingMapper.toDto(reviewRating);
    }

    /**
     * Partially update a reviewRating.
     *
     * @param reviewRatingDTO the entity to update partially.
     * @return the persisted entity.
     */
    public Optional<ReviewRatingDTO> partialUpdate(ReviewRatingDTO reviewRatingDTO) {
        log.debug("Request to partially update ReviewRating : {}", reviewRatingDTO);

        return reviewRatingRepository
            .findById(reviewRatingDTO.getId())
            .map(existingReviewRating -> {
                reviewRatingMapper.partialUpdate(existingReviewRating, reviewRatingDTO);

                return existingReviewRating;
            })
            .map(reviewRatingRepository::save)
            .map(reviewRatingMapper::toDto);
    }

    /**
     * Get all the reviewRatings.
     *
     * @return the list of entities.
     */
    @Transactional(readOnly = true)
    public List<ReviewRatingDTO> findAll() {
        log.debug("Request to get all ReviewRatings");
        return reviewRatingRepository.findAll().stream().map(reviewRatingMapper::toDto).collect(Collectors.toCollection(LinkedList::new));
    }

    /**
     * Get one reviewRating by id.
     *
     * @param id the id of the entity.
     * @return the entity.
     */
    @Transactional(readOnly = true)
    public Optional<ReviewRatingDTO> findOne(Long id) {
        log.debug("Request to get ReviewRating : {}", id);
        return reviewRatingRepository.findById(id).map(reviewRatingMapper::toDto);
    }

    /**
     * Delete the reviewRating by id.
     *
     * @param id the id of the entity.
     */
    public void delete(Long id) {
        log.debug("Request to delete ReviewRating : {}", id);
        reviewRatingRepository.deleteById(id);
    }
}
