import { Injectable } from '@angular/core';
import { HttpClient, HttpResponse } from '@angular/common/http';
import { Observable } from 'rxjs';

import { isPresent } from 'app/core/util/operators';
import { ApplicationConfigService } from 'app/core/config/application-config.service';
import { createRequestOption } from 'app/core/request/request-util';
import { ISavedSearches, NewSavedSearches } from '../saved-searches.model';

export type PartialUpdateSavedSearches = Partial<ISavedSearches> & Pick<ISavedSearches, 'id'>;

export type EntityResponseType = HttpResponse<ISavedSearches>;
export type EntityArrayResponseType = HttpResponse<ISavedSearches[]>;

@Injectable({ providedIn: 'root' })
export class SavedSearchesService {
  protected resourceUrl = this.applicationConfigService.getEndpointFor('api/saved-searches');

  constructor(protected http: HttpClient, protected applicationConfigService: ApplicationConfigService) {}

  create(savedSearches: NewSavedSearches): Observable<EntityResponseType> {
    return this.http.post<ISavedSearches>(this.resourceUrl, savedSearches, { observe: 'response' });
  }

  update(savedSearches: ISavedSearches): Observable<EntityResponseType> {
    return this.http.put<ISavedSearches>(`${this.resourceUrl}/${this.getSavedSearchesIdentifier(savedSearches)}`, savedSearches, {
      observe: 'response',
    });
  }

  partialUpdate(savedSearches: PartialUpdateSavedSearches): Observable<EntityResponseType> {
    return this.http.patch<ISavedSearches>(`${this.resourceUrl}/${this.getSavedSearchesIdentifier(savedSearches)}`, savedSearches, {
      observe: 'response',
    });
  }

  find(id: number): Observable<EntityResponseType> {
    return this.http.get<ISavedSearches>(`${this.resourceUrl}/${id}`, { observe: 'response' });
  }

  query(req?: any): Observable<EntityArrayResponseType> {
    const options = createRequestOption(req);
    return this.http.get<ISavedSearches[]>(this.resourceUrl, { params: options, observe: 'response' });
  }

  delete(id: number): Observable<HttpResponse<{}>> {
    return this.http.delete(`${this.resourceUrl}/${id}`, { observe: 'response' });
  }

  getSavedSearchesIdentifier(savedSearches: Pick<ISavedSearches, 'id'>): number {
    return savedSearches.id;
  }

  compareSavedSearches(o1: Pick<ISavedSearches, 'id'> | null, o2: Pick<ISavedSearches, 'id'> | null): boolean {
    return o1 && o2 ? this.getSavedSearchesIdentifier(o1) === this.getSavedSearchesIdentifier(o2) : o1 === o2;
  }

  addSavedSearchesToCollectionIfMissing<Type extends Pick<ISavedSearches, 'id'>>(
    savedSearchesCollection: Type[],
    ...savedSearchesToCheck: (Type | null | undefined)[]
  ): Type[] {
    const savedSearches: Type[] = savedSearchesToCheck.filter(isPresent);
    if (savedSearches.length > 0) {
      const savedSearchesCollectionIdentifiers = savedSearchesCollection.map(
        savedSearchesItem => this.getSavedSearchesIdentifier(savedSearchesItem)!
      );
      const savedSearchesToAdd = savedSearches.filter(savedSearchesItem => {
        const savedSearchesIdentifier = this.getSavedSearchesIdentifier(savedSearchesItem);
        if (savedSearchesCollectionIdentifiers.includes(savedSearchesIdentifier)) {
          return false;
        }
        savedSearchesCollectionIdentifiers.push(savedSearchesIdentifier);
        return true;
      });
      return [...savedSearchesToAdd, ...savedSearchesCollection];
    }
    return savedSearchesCollection;
  }
}
