import { Injectable } from '@angular/core';
import { HttpClient, HttpResponse } from '@angular/common/http';
import { Observable } from 'rxjs';

import { isPresent } from 'app/core/util/operators';
import { ApplicationConfigService } from 'app/core/config/application-config.service';
import { createRequestOption } from 'app/core/request/request-util';
import { IWatchListEntry, getWatchListEntryIdentifier } from '../watch-list-entry.model';

export type EntityResponseType = HttpResponse<IWatchListEntry>;
export type EntityArrayResponseType = HttpResponse<IWatchListEntry[]>;

@Injectable({ providedIn: 'root' })
export class WatchListEntryService {
  protected resourceUrl = this.applicationConfigService.getEndpointFor('api/watch-list-entries');

  constructor(protected http: HttpClient, protected applicationConfigService: ApplicationConfigService) {}

  create(watchListEntry: IWatchListEntry): Observable<EntityResponseType> {
    return this.http.post<IWatchListEntry>(this.resourceUrl, watchListEntry, { observe: 'response' });
  }

  update(watchListEntry: IWatchListEntry): Observable<EntityResponseType> {
    return this.http.put<IWatchListEntry>(`${this.resourceUrl}/${getWatchListEntryIdentifier(watchListEntry) as number}`, watchListEntry, {
      observe: 'response',
    });
  }

  partialUpdate(watchListEntry: IWatchListEntry): Observable<EntityResponseType> {
    return this.http.patch<IWatchListEntry>(
      `${this.resourceUrl}/${getWatchListEntryIdentifier(watchListEntry) as number}`,
      watchListEntry,
      { observe: 'response' }
    );
  }

  find(id: number): Observable<EntityResponseType> {
    return this.http.get<IWatchListEntry>(`${this.resourceUrl}/${id}`, { observe: 'response' });
  }

  query(req?: any): Observable<EntityArrayResponseType> {
    const options = createRequestOption(req);
    return this.http.get<IWatchListEntry[]>(this.resourceUrl, { params: options, observe: 'response' });
  }

  delete(id: number): Observable<HttpResponse<{}>> {
    return this.http.delete(`${this.resourceUrl}/${id}`, { observe: 'response' });
  }

  addWatchListEntryToCollectionIfMissing(
    watchListEntryCollection: IWatchListEntry[],
    ...watchListEntriesToCheck: (IWatchListEntry | null | undefined)[]
  ): IWatchListEntry[] {
    const watchListEntries: IWatchListEntry[] = watchListEntriesToCheck.filter(isPresent);
    if (watchListEntries.length > 0) {
      const watchListEntryCollectionIdentifiers = watchListEntryCollection.map(
        watchListEntryItem => getWatchListEntryIdentifier(watchListEntryItem)!
      );
      const watchListEntriesToAdd = watchListEntries.filter(watchListEntryItem => {
        const watchListEntryIdentifier = getWatchListEntryIdentifier(watchListEntryItem);
        if (watchListEntryIdentifier == null || watchListEntryCollectionIdentifiers.includes(watchListEntryIdentifier)) {
          return false;
        }
        watchListEntryCollectionIdentifiers.push(watchListEntryIdentifier);
        return true;
      });
      return [...watchListEntriesToAdd, ...watchListEntryCollection];
    }
    return watchListEntryCollection;
  }
}
